@@ -28,10 +28,17 @@ Unit tests for _grid_actions.py
 
 """
 
+import bz2
 import os
 import sys
 
-import bz2
+try:
+    import gnupg
+except ImportError:
+    gnupg = None
+
+import pytest
+
 import wx
 app = wx.App()
 
@@ -48,11 +55,22 @@ from src.lib.testlib import basic_setup_test, restore_basic_grid
 
 from src.gui._events import *
 
+try:
+    from src.lib.gpg import genkey
+except ImportError:
+    genkey = None
+
+from src.config import config
 
 class TestFileActions(object):
     """File actions test class"""
 
     def setup_method(self, method):
+
+        # Generate a GPG key if not present
+        if genkey is not None:
+            self.fingerprint = genkey(key_name="pyspread_test_key")
+
         self.main_window = MainWindow(None, title="pyspread", S=None)
         self.grid = self.main_window.grid
         self.code_array = self.grid.code_array
@@ -86,6 +104,17 @@ class TestFileActions(object):
         # File for testing save
         self.filename_save = TESTPATH + "test_save.pys"
 
+    def teardown_method(self, method):
+        if gnupg is not None and \
+           self.fingerprint != config["gpg_key_fingerprint"] and \
+           self.fingerprint is not None:
+            gpg = gnupg.GPG()
+            print repr(self.fingerprint)
+            # Secret key must be deleted first
+            gpg.delete_keys(self.fingerprint, True)
+            gpg.delete_keys(self.fingerprint)
+
+    @pytest.mark.skipif(gnupg is None, reason="requires gnupg")
     def test_validate_signature(self):
         """Tests signature validation"""
 
@@ -113,6 +142,7 @@ class TestFileActions(object):
         self.grid.actions.leave_safe_mode()
         assert not self.grid.code_array.safe_mode
 
+    @pytest.mark.skipif(gnupg is None, reason="requires gnupg")
     def test_approve(self):
 
         # Test if safe_mode is correctly set for invalid sig
@@ -240,34 +270,35 @@ class TestFileActions(object):
 
         assert self.grid.GetTable().data_array.safe_mode  # sig is also empty
 
-        # Test invalid sig files
-        event.attr["filepath"] = self.filename_invalid_sig
-        self.grid.actions.open(event)
+        if gnupg is not None:
+            # Test invalid sig files
+            event.attr["filepath"] = self.filename_invalid_sig
+            self.grid.actions.open(event)
 
-        assert self.grid.GetTable().data_array.safe_mode
+            assert self.grid.GetTable().data_array.safe_mode
 
-        # Test file with sig
-        event.attr["filepath"] = self.filename_valid_sig
-        self.grid.actions.open(event)
+            # Test file with sig
+            event.attr["filepath"] = self.filename_valid_sig
+            self.grid.actions.open(event)
 
-        assert not self.grid.GetTable().data_array.safe_mode
+            assert not self.grid.GetTable().data_array.safe_mode
 
-        # Test file without sig
-        event.attr["filepath"] = self.filename_no_sig
-        self.grid.actions.open(event)
+            # Test file without sig
+            event.attr["filepath"] = self.filename_no_sig
+            self.grid.actions.open(event)
 
-        assert self.grid.GetTable().data_array.safe_mode
+            assert self.grid.GetTable().data_array.safe_mode
 
-        # Test self.grid size for valid file
-        event.attr["filepath"] = self.filename_gridsize
-        self.grid.actions.open(event)
+            # Test self.grid size for valid file
+            event.attr["filepath"] = self.filename_gridsize
+            self.grid.actions.open(event)
 
-        new_shape = self.grid.GetTable().data_array.shape
-        assert new_shape == (1000, 100, 10)
+            new_shape = self.grid.GetTable().data_array.shape
+            assert new_shape == (1000, 100, 10)
 
-        # Test self.grid content for valid file
-        assert not self.grid.code_array.safe_mode
-        assert self.grid.GetTable().data_array[0, 0, 0] == "test4"
+            # Test self.grid content for valid file
+            assert not self.grid.code_array.safe_mode
+            assert self.grid.GetTable().data_array[0, 0, 0] == "test4"
 
     def test_save(self):
         """Tests save functionality"""
@@ -309,15 +340,17 @@ class TestFileActions(object):
         except TypeError:
             pass
 
-        # Test sig creation is happening
+        if gnupg is not None:
+            # Test sig creation is happening
 
-        sigfile = open(self.filename_save + ".sig")
-        assert sigfile
-        sigfile.close()
+            sigfile = open(self.filename_save + ".sig")
+            assert sigfile
+            sigfile.close()
+            os.remove(self.filename_save + ".sig")
 
         os.remove(self.filename_save)
-        os.remove(self.filename_save + ".sig")
 
+    @pytest.mark.skipif(gnupg is None, reason="requires gnupg")
     def test_sign_file(self):
         """Tests signing functionality"""
 
@@ -724,7 +757,6 @@ class TestTableActions(object):
         for __key in data:
             self.grid.code_array[__key] = data[__key]
 
-
         selection.grid_select(self.grid)
 
         try:
@@ -889,15 +921,16 @@ class TestSelectionActions(object):
         """Tests for quote_selection"""
 
         self.grid.code_array[(1, 0, 0)] = "Q1"
-        self.grid.code_array[(2, 0, 0)] = '"NQ1"'
+        self.grid.code_array[(2, 0, 0)] = 'u"NQ1"'
 
         self.grid.actions.select_cell(1, 0)
         self.grid.actions.select_cell(2, 0, add_to_selected=True)
 
         self.grid.actions.quote_selection()
 
-        assert self.grid.code_array((1, 0, 0)) == '"Q1"'
-        assert self.grid.code_array((2, 0, 0)) == '"NQ1"'
+        assert self.grid.code_array((1, 0, 0)) == 'u"Q1"'
+        assert self.grid.code_array((2, 0, 0)) == 'u"NQ1"'
+
 
 class TestFindActions(object):
     """FindActions test class"""
