@@ -67,7 +67,8 @@ import ast
 from traceback import print_exception
 from StringIO import StringIO
 from sys import exc_info
-#use ugettext instead of getttext to avoid unicode errors
+
+# use ugettext instead of gettext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -304,7 +305,7 @@ class CsvParameterWidgets(object):
             handler = getattr(self, self.widget_handlers[pname])
             self.parent.Bind(event_type, handler, widget)
 
-            #Tool tips
+            # Tool tips
             label.SetToolTipString(phelp)
             widget.SetToolTipString(phelp)
 
@@ -392,7 +393,6 @@ class CsvParameterWidgets(object):
         else:
             dialect = csv.get_dialect(dialect_name)
 
-        #print dialect, self.has_header
         self._update_settings(dialect)
 
         self.choice_dialects.SetValue(value)
@@ -487,7 +487,6 @@ class CSVPreviewGrid(wx.grid.Grid):
     def _set_properties(self):
         self.SetRowLabelSize(0)
         self.SetColLabelSize(0)
-        #self.EnableEditing(0)
         self.EnableDragGridSize(0)
 
     def OnGridEditorCreated(self, event):
@@ -1167,7 +1166,7 @@ class AboutDialog(object):
             350, wx.ClientDC(parent))
         info.WebSite = ("http://manns.github.io/pyspread/",
                         _("Pyspread Web site"))
-        info.Developers = ["Martin Manns"]
+        info.Developers = ["Martin Manns", "Jason Sexauer", "Vova Kolobok"]
         info.DocWriters = ["Martin Manns", "Bosko Markovic"]
         info.Translators = ["Joe Hansen", "Mark Haanen", "Yuri Chornoivan",
                             u"Mario Blättermann", "Christian Kirbach",
