@@ -208,11 +208,12 @@ class MainWindow(wx.Frame, EventMixin):
 
             # Get menu item to toggle
             toggle_id = self.menubar.FindMenuItem(_("View"), toggle_label)
+            if toggle_id != -1:
+                # Check may fail if translation is incomplete
+                toggle_item = self.menubar.FindItemById(toggle_id)
 
-            toggle_item = self.menubar.FindItemById(toggle_id)
-
-            # Adjust toggle to pane visibility
-            toggle_item.Check(pane.IsShown())
+                # Adjust toggle to pane visibility
+                toggle_item.Check(pane.IsShown())
 
     def _do_layout(self):
         """Adds widgets to the aui manager and controls the layout"""
@@ -280,6 +281,7 @@ class MainWindow(wx.Frame, EventMixin):
         # Main window events
         self.Bind(wx.EVT_MOVE, handlers.OnMove)
         self.Bind(wx.EVT_SIZE, handlers.OnSize)
+        self.Bind(self.EVT_CMD_FULLSCREEN, handlers.OnToggleFullscreen)
 
         # Content changed event, adjusts title bar with star
         self.Bind(self.EVT_CONTENT_CHANGED, handlers.OnContentChanged)
@@ -407,6 +409,44 @@ class MainWindowEventHandlers(EventMixin):
 
         config["window_size"] = repr((size.width, size.height))
 
+    def OnToggleFullscreen(self, event):
+        """Fullscreen event handler"""
+
+        is_full_screen = self.main_window.IsFullScreen()
+
+        # Make sure that only the grid is shown in fullscreen mode
+        if is_full_screen:
+            try:
+                self.main_window.grid.SetRowLabelSize(self.row_label_size)
+                self.main_window.grid.SetColLabelSize(self.col_label_size)
+            except AttributeError:
+                pass
+
+            # Restore toolbars from before fullscreen mode
+            try:
+                self.main_window._mgr.LoadPerspective(self.aui_windowed)
+            except AttributeError:
+                pass
+
+        else:
+            # Save the perspective of the AUI manager
+            self.aui_windowed = self.main_window._mgr.SavePerspective()
+
+            # Hide toolbars
+            for pane in self.main_window._mgr.GetAllPanes():
+                if pane.name != "grid":
+                    pane.Hide()
+
+            self.main_window._mgr.Update()
+
+            self.row_label_size = self.main_window.grid.GetRowLabelSize()
+            self.col_label_size = self.main_window.grid.GetColLabelSize()
+
+            self.main_window.grid.HideRowLabels()
+            self.main_window.grid.HideColLabels()
+
+        self.main_window.ShowFullScreen(not is_full_screen)
+
     def OnContentChanged(self, event):
         """Titlebar star adjustment event handler"""
 
@@ -998,6 +1038,9 @@ class MainWindowEventHandlers(EventMixin):
             self.interfaces.get_filepath_findex_from_user(wildcard, message,
                                                           style)
 
+        if path is None:
+            return
+
         # If an single cell is exported then the selection bbox
         # has to be changed to the current cell
         if filters[filterindex].startswith("cell_"):
@@ -1363,6 +1406,9 @@ class MainWindowEventHandlers(EventMixin):
             self.interfaces.get_filepath_findex_from_user(wildcard, message,
                                                           style)
 
+        if filepath is None:
+            return
+
         # Save macros to file
 
         macros = self.main_window.grid.code_array.macros
@@ -1376,7 +1422,7 @@ class MainWindowEventHandlers(EventMixin):
         """Manual launch event handler"""
 
         self.main_window.actions.launch_help("First steps in pyspread",
-                                             "First steps.html")
+                                             "first_steps.html")
 
     def OnTutorial(self, event):
         """Tutorial launch event handler"""
