@@ -37,8 +37,10 @@ Provides
 import math
 import sys
 import warnings
+from operator import attrgetter
 
 import cairo
+import numpy
 import wx
 import wx.lib.wxcairo
 
@@ -97,7 +99,8 @@ class GridCairoRenderer(object):
     """
 
     def __init__(self, context, code_array, row_tb, col_rl, tab_fl,
-                 width, height, orientation, x_offset=20.5, y_offset=20.5):
+                 width, height, orientation, x_offset=20.5, y_offset=20.5,
+                 view_frozen=False):
         self.context = context
         self.code_array = code_array
 
@@ -113,6 +116,8 @@ class GridCairoRenderer(object):
 
         self.orientation = orientation
 
+        self.view_frozen = view_frozen
+
     def get_cell_rect(self, row, col, tab):
         """Returns rectangle of cell on canvas"""
 
@@ -219,7 +224,8 @@ class GridCairoRenderer(object):
                             self.context,
                             self.code_array,
                             key,  # (row, col, tab)
-                            rect
+                            rect,
+                            self.view_frozen
                         )
 
                         cell_renderer.draw()
@@ -247,11 +253,12 @@ class GridCellCairoRenderer(object):
 
     """
 
-    def __init__(self, context, code_array, key, rect):
+    def __init__(self, context, code_array, key, rect, view_frozen=False):
         self.context = context
         self.code_array = code_array
         self.key = key
         self.rect = rect
+        self.view_frozen = view_frozen
 
     def draw(self):
         """Draws cell to context"""
@@ -260,7 +267,8 @@ class GridCellCairoRenderer(object):
             self.context,
             self.code_array,
             self.key,
-            self.rect)
+            self.rect,
+            self.view_frozen)
 
         cell_content_renderer = GridCellContentCairoRenderer(
             self.context,
@@ -316,11 +324,114 @@ class GridCellContentCairoRenderer(object):
         except IndexError:
             pass
 
+    def _get_scalexy(self, ims_width, ims_height):
+        """Returns scale_x, scale_y for bitmap display"""
+
+        # Get cell attributes
+        cell_attributes = self.code_array.cell_attributes[self.key]
+        angle = cell_attributes["angle"]
+
+        if abs(angle) == 90:
+            scale_x = self.rect[3] / float(ims_width)
+            scale_y = self.rect[2] / float(ims_height)
+
+        else:
+            # Normal case
+            scale_x = self.rect[2] / float(ims_width)
+            scale_y = self.rect[3] / float(ims_height)
+
+        return scale_x, scale_y
+
+    def _get_translation(self, ims_width, ims_height):
+        """Returns x and y for a bitmap translation"""
+
+        # Get cell attributes
+        cell_attributes = self.code_array.cell_attributes[self.key]
+        justification = cell_attributes["justification"]
+        vertical_align = cell_attributes["vertical_align"]
+        angle = cell_attributes["angle"]
+
+        scale_x, scale_y = self._get_scalexy(ims_width, ims_height)
+        scale = min(scale_x, scale_y)
+
+        if angle not in (90, 180, -90):
+            # Standard direction
+            x = -2  # Otherwise there is a white border
+            y = -2  # Otherwise there is a white border
+
+            if scale_x > scale_y:
+                if justification == "center":
+                    x += (self.rect[2] - ims_width * scale) / 2
+
+                elif justification == "right":
+                    x += self.rect[2] - ims_width * scale
+
+            else:
+                if vertical_align == "middle":
+                    y += (self.rect[3] - ims_height * scale) / 2
+
+                elif vertical_align == "bottom":
+                    y += self.rect[3] - ims_height * scale
+
+        if angle == 90:
+            x = -ims_width * scale + 2
+            y = -2
+
+            if scale_y > scale_x:
+                if justification == "center":
+                    y += (self.rect[2] - ims_height * scale) / 2
+
+                elif justification == "right":
+                    y += self.rect[2] - ims_height * scale
+
+            else:
+                if vertical_align == "middle":
+                    x -= (self.rect[3] - ims_width * scale) / 2
+
+                elif vertical_align == "bottom":
+                    x -= self.rect[3] - ims_width * scale
+
+        elif angle == 180:
+            x = -ims_width * scale + 2
+            y = -ims_height * scale + 2
+
+            if scale_x > scale_y:
+                if justification == "center":
+                    x -= (self.rect[2] - ims_width * scale) / 2
+
+                elif justification == "right":
+                    x -= self.rect[2] - ims_width * scale
+
+            else:
+                if vertical_align == "middle":
+                    y -= (self.rect[3] - ims_height * scale) / 2
+
+                elif vertical_align == "bottom":
+                    y -= self.rect[3] - ims_height * scale
+
+        elif angle == -90:
+            x = -2
+            y = -ims_height * scale + 2
+
+            if scale_y > scale_x:
+                if justification == "center":
+                    y -= (self.rect[2] - ims_height * scale) / 2
+
+                elif justification == "right":
+                    y -= self.rect[2] - ims_height * scale
+
+            else:
+                if vertical_align == "middle":
+                    x += (self.rect[3] - ims_width * scale) / 2
+
+                elif vertical_align == "bottom":
+                    x += self.rect[3] - ims_width * scale
+
+        return x, y
+
     def draw_bitmap(self, content):
         """Draws bitmap cell content to context"""
 
-        # Get
-
         if content.HasAlpha():
             image = wx.ImageFromBitmap(content)
             image.ConvertAlphaToMask()
@@ -328,18 +439,21 @@ class GridCellContentCairoRenderer(object):
             content = wx.BitmapFromImage(image)
 
         ims = wx.lib.wxcairo.ImageSurfaceFromBitmap(content)
+
         ims_width = ims.get_width()
         ims_height = ims.get_height()
 
-        scale_x = self.rect[2] / float(ims_width)
-        scale_y = self.rect[3] / float(ims_height)
+        transx, transy = self._get_translation(ims_width, ims_height)
 
-        scale_x = min(scale_x, scale_y)
-        scale_y = scale_x
+        scale_x, scale_y = self._get_scalexy(ims_width, ims_height)
+        scale = min(scale_x, scale_y)
+
+        angle = float(self.code_array.cell_attributes[self.key]["angle"])
 
         self.context.save()
-        self.context.translate(-2, -2)  # Otherwise there is a white border
-        self.context.scale(scale_x, scale_y)
+        self.context.rotate(-angle / 360 * 2 * math.pi)
+        self.context.translate(transx, transy)
+        self.context.scale(scale, scale)
         self.context.set_source_surface(ims, 0, 0)
         self.context.paint()
         self.context.restore()
@@ -355,13 +469,19 @@ class GridCellContentCairoRenderer(object):
 
         svg = rsvg.Handle(data=svg_str)
 
-        dim = svg.get_dimension_data()
-        scale_x = self.rect[2] / float(dim[0] + 10)  # Avoid cut offs
-        scale_y = self.rect[3] / float(dim[1] + 5)
-        scale_x = min(scale_x, scale_y)
-        scale_y = scale_x
+        svg_width, svg_height = svg.get_dimension_data()[:2]
+
+        transx, transy = self._get_translation(svg_width, svg_height)
+
+        scale_x, scale_y = self._get_scalexy(svg_width, svg_height)
+        scale = min(scale_x, scale_y)
+
+        angle = float(self.code_array.cell_attributes[self.key]["angle"])
+
         self.context.save()
-        self.context.scale(scale_x, scale_y)
+        self.context.rotate(-angle / 360 * 2 * math.pi)
+        self.context.translate(transx, transy)
+        self.context.scale(scale, scale)
         svg.render_cairo(self.context)
         self.context.restore()
 
@@ -575,14 +695,15 @@ class GridCellContentCairoRenderer(object):
         elif cell_attributes["vertical_align"] == "middle":
             downshift = int((rect[3] - extents[1][3]) / 2) - 2
 
+        self.context.save()
+
         self._rotate_cell(angle, rect)
         self.context.translate(0, downshift)
 
         ptx.update_layout(pango_layout)
         ptx.show_layout(pango_layout)
 
-        self.context.translate(0, -downshift)
-        self._rotate_cell(angle, rect, back=True)
+        self.context.restore()
 
     def draw_roundedrect(self, x, y, w, h, r=10):
         """Draws a rounded rectangle"""
@@ -663,6 +784,16 @@ class GridCellContentCairoRenderer(object):
         pos_x, pos_y = self.rect[:2]
         self.context.translate(pos_x + 2, pos_y + 2)
 
+        cell_attributes = self.code_array.cell_attributes
+
+        # Do not draw cell content if cell is too small
+        # This allows blending out small cells by reducing height to 0
+
+        if self.rect[2] < cell_attributes[self.key]["borderwidth_right"] or \
+           self.rect[3] < cell_attributes[self.key]["borderwidth_bottom"]:
+            self.context.restore()
+            return
+
         if self.code_array.cell_attributes[self.key]["button_cell"]:
             # Render a button instead of the cell
             label = self.code_array.cell_attributes[self.key]["button_cell"]
@@ -677,7 +808,7 @@ class GridCellContentCairoRenderer(object):
             self.draw_matplotlib_figure(content)
 
         elif isinstance(content, basestring) and is_svg(content):
-            # The content
+            # The content is a vaid SVG xml string
             self.draw_svg(content)
 
         elif content is not None:
@@ -701,14 +832,17 @@ class GridCellBackgroundCairoRenderer(object):
     \tGrid data structure that yields rendering information
     * key: 3 tuple of Integer
     \tKey of cell to be rendered
+    * view_frozen: Bool
+    \tIf true then paint frozen background pattern for frozen cells
 
     """
 
-    def __init__(self, context, code_array, key, rect):
+    def __init__(self, context, code_array, key, rect, view_frozen):
         self.context = context
         self.cell_attributes = code_array.cell_attributes
         self.key = key
         self.rect = rect
+        self.view_frozen = view_frozen
 
     def _get_background_color(self):
         """Returns background color rgb tuple of right line"""
@@ -716,6 +850,24 @@ class GridCellBackgroundCairoRenderer(object):
         color = self.cell_attributes[self.key]["bgcolor"]
         return tuple(c / 255.0 for c in color_pack2rgb(color))
 
+    def _draw_frozen_pattern(self):
+        """Draws frozen pattern, i.e. diagonal lines"""
+
+        self.context.save()
+
+        x, y, w, h = self.rect
+        self.context.set_source_rgb(0, 0, 1)
+        self.context.set_line_width(0.25)
+        self.context.rectangle(*self.rect)
+        self.context.clip()
+
+        for __x in numpy.arange(x-h, x+w, 5):
+            self.context.move_to(__x, y + h)
+            self.context.line_to(__x + h, y)
+        self.context.stroke()
+
+        self.context.restore()
+
     def draw(self):
         """Draws cell background to context"""
 
@@ -723,41 +875,187 @@ class GridCellBackgroundCairoRenderer(object):
         self.context.rectangle(*self.rect)
         self.context.fill()
 
+        # If show frozen is active, show frozen pattern
+        if self.view_frozen and self.cell_attributes[self.key]["frozen"]:
+            self._draw_frozen_pattern()
 
-class GridCellBorderCairoRenderer(object):
-    """Renders cell border to Cairo context
+
+class CellBorder(object):
+    """Cell border
 
     Parameters
     ----------
-
-    * context: pycairo.context
-    \tThe Cairo context to be drawn to
-    * code_array: model.code_array
-    \tGrid data structure that yields rendering information
-    * key: 3 tuple of Integer
-    \tKey of cell to be rendered
+    start_point: 2 tuple of Integer
+    \tStart point of line
+    end_point
+    \tEnd point of line
+    width: Float
+    \tWidth of line
+    color: 3-tuple of float
+    \tRGB line color, each value is in [0, 1]
 
     """
 
-    def __init__(self, context, code_array, key, rect):
-        self.context = context
-        self.code_array = code_array
-        self.cell_attributes = code_array.cell_attributes
+    def __init__(self, start_point, end_point, width, color):
+        self.start_point = start_point
+        self.end_point = end_point
+        self.width = width
+        self.color = color
+
+    def draw(self, context):
+        """Draws self to Cairo context"""
+
+        context.set_line_width(self.width)
+        context.set_source_rgb(*self.color)
+
+        context.move_to(*self.start_point)
+        context.line_to(*self.end_point)
+        context.stroke()
+
+
+class Cell(object):
+    """Cell"""
+
+    def __init__(self, key, rect, cell_attributes):
+        self.row, self.col, self.tab = self.key = key
+        self.x, self.y, self.width, self.height = rect
+        self.cell_attributes = cell_attributes
+
+    def get_above_key_rect(self):
+        """Returns tuple key rect of above cell"""
+
+        key_above = self.row - 1, self.col, self.tab
+
+        border_width_bottom = \
+            float(self.cell_attributes[key_above]["borderwidth_bottom"]) / 2.0
+
+        rect_above = (self.x, self.y-border_width_bottom,
+                      self.width, border_width_bottom)
+        return key_above, rect_above
+
+    def get_below_key_rect(self):
+        """Returns tuple key rect of below cell"""
+
+        key_below = self.row + 1, self.col, self.tab
+
+        border_width_bottom = \
+            float(self.cell_attributes[self.key]["borderwidth_bottom"]) / 2.0
+
+        rect_below = (self.x, self.y+self.height,
+                      self.width, border_width_bottom)
+        return key_below, rect_below
+
+    def get_left_key_rect(self):
+        """Returns tuple key rect of left cell"""
+
+        key_left = self.row, self.col - 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[key_left]["borderwidth_right"]) / 2.0
+
+        rect_left = (self.x-border_width_right, self.y,
+                     border_width_right, self.height)
+        return key_left, rect_left
+
+    def get_right_key_rect(self):
+        """Returns tuple key rect of right cell"""
+
+        key_right = self.row, self.col + 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[self.key]["borderwidth_right"]) / 2.0
+
+        rect_right = (self.x+self.width, self.y,
+                      border_width_right, self.height)
+        return key_right, rect_right
+
+    def get_above_left_key_rect(self):
+        """Returns tuple key rect of above left cell"""
+
+        key_above_left = self.row - 1, self.col - 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[key_above_left]["borderwidth_right"]) \
+            / 2.0
+        border_width_bottom = \
+            float(self.cell_attributes[key_above_left]["borderwidth_bottom"]) \
+            / 2.0
+
+        rect_above_left = (self.x-border_width_right,
+                           self.y-border_width_bottom,
+                           border_width_right, border_width_bottom)
+        return key_above_left, rect_above_left
+
+    def get_above_right_key_rect(self):
+        """Returns tuple key rect of above right cell"""
+
+        key_above = self.row - 1, self.col, self.tab
+        key_above_right = self.row - 1, self.col + 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[key_above]["borderwidth_right"]) / 2.0
+        border_width_bottom = \
+            float(self.cell_attributes[key_above_right]["borderwidth_bottom"])\
+            / 2.0
+
+        rect_above_right = (self.x+self.width, self.y-border_width_bottom,
+                            border_width_right, border_width_bottom)
+        return key_above_right, rect_above_right
+
+    def get_below_left_key_rect(self):
+        """Returns tuple key rect of below left cell"""
+
+        key_left = self.row, self.col - 1, self.tab
+        key_below_left = self.row + 1, self.col - 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[key_below_left]["borderwidth_right"]) \
+            / 2.0
+        border_width_bottom = \
+            float(self.cell_attributes[key_left]["borderwidth_bottom"]) / 2.0
+
+        rect_below_left = (self.x-border_width_right, self.y-self.height,
+                           border_width_right, border_width_bottom)
+        return key_below_left, rect_below_left
+
+    def get_below_right_key_rect(self):
+        """Returns tuple key rect of below right cell"""
+
+        key_below_right = self.row + 1, self.col + 1, self.tab
+
+        border_width_right = \
+            float(self.cell_attributes[self.key]["borderwidth_right"]) / 2.0
+        border_width_bottom = \
+            float(self.cell_attributes[self.key]["borderwidth_bottom"]) / 2.0
+
+        rect_below_right = (self.x+self.width, self.y-self.height,
+                            border_width_right, border_width_bottom)
+        return key_below_right, rect_below_right
+
+
+class CellBorders(object):
+    """All 12 relevant borders around a cell
+
+       tl   tr
+        | t |
+    lt -|---|- rt
+        |l r|
+    lb -|---|- rb
+        | b |
+       bl   br
+
+    Parameters
+    ----------
+    key: 3 tuple
+    \tCell key
+
+    """
+
+    def __init__(self, cell_attributes, key, rect):
         self.key = key
         self.rect = rect
-
-    # Coordinates
-    # -----------
-
-    def _get_top_line_coordinates(self):
-        """Returns start and stop coordinates of bottom line"""
-
-        rect_x, rect_y, rect_width, _ = self.rect
-
-        start_point = rect_x, rect_y
-        end_point = rect_x + rect_width, rect_y
-
-        return start_point, end_point
+        self.cell_attributes = cell_attributes
+        self.cell = Cell(key, rect, cell_attributes)
 
     def _get_bottom_line_coordinates(self):
         """Returns start and stop coordinates of bottom line"""
@@ -769,16 +1067,6 @@ class GridCellBorderCairoRenderer(object):
 
         return start_point, end_point
 
-    def _get_left_line_coordinates(self):
-        """Returns start and stop coordinates of right line"""
-
-        rect_x, rect_y, _, rect_height = self.rect
-
-        start_point = rect_x, rect_y
-        end_point = rect_x, rect_y + rect_height
-
-        return start_point, end_point
-
     def _get_right_line_coordinates(self):
         """Returns start and stop coordinates of right line"""
 
@@ -789,174 +1077,150 @@ class GridCellBorderCairoRenderer(object):
 
         return start_point, end_point
 
-    def _get_topleft_right_line_coordinates(self):
-        """Returns start and stop coordinates of top left cell's right line"""
-
-        rect_x, rect_y, _, _ = self.rect
-
-        start_point = rect_x, rect_y - 0.01
-        end_point = rect_x, rect_y
-
-        return start_point, end_point
-
-    def _get_topleft_bottom_line_coordinates(self):
-        """Returns start and stop coordinates of top left cell's bottom line"""
-
-        rect_x, rect_y, _, _ = self.rect
-
-        start_point = rect_x - 0.01, rect_y
-        end_point = rect_x, rect_y
-
-        return start_point, end_point
-
-    # Colors
-    # ------
-
-    def _get_top_line_color(self):
-        """Returns color rgb tuple of top line"""
-
-        row, col, tab = self.key
-        key = row - 1, col, tab
-        color = self.cell_attributes[key]["bordercolor_bottom"]
-        return tuple(c / 255.0 for c in color_pack2rgb(color))
-
     def _get_bottom_line_color(self):
         """Returns color rgb tuple of bottom line"""
 
         color = self.cell_attributes[self.key]["bordercolor_bottom"]
         return tuple(c / 255.0 for c in color_pack2rgb(color))
 
-    def _get_left_line_color(self):
-        """Returns color rgb tuple of left line"""
-
-        row, col, tab = self.key
-        key = row, col - 1, tab
-        color = self.cell_attributes[key]["bordercolor_right"]
-        return tuple(c / 255.0 for c in color_pack2rgb(color))
-
     def _get_right_line_color(self):
         """Returns color rgb tuple of right line"""
 
         color = self.cell_attributes[self.key]["bordercolor_right"]
         return tuple(c / 255.0 for c in color_pack2rgb(color))
 
-    def _get_topleft_right_line_color(self):
-        """Returns color rgb tuple of right line of top left cell"""
-
-        row, col, tab = self.key
-        key = row - 1, col - 1, tab
-        color = self.cell_attributes[key]["bordercolor_right"]
-        return tuple(c / 255.0 for c in color_pack2rgb(color))
-
-    def _get_topleft_bottom_line_color(self):
-        """Returns color rgb tuple of bottom line of top left cell"""
-
-        row, col, tab = self.key
-        key = row - 1, col - 1, tab
-        color = self.cell_attributes[key]["bordercolor_bottom"]
-        return tuple(c / 255.0 for c in color_pack2rgb(color))
-
-    # Widths
-    # ------
-
-    def _get_top_line_width(self):
-        """Returns width of top line"""
-
-        row, col, tab = self.key
-        key = row - 1, col, tab
-        return float(self.cell_attributes[key]["borderwidth_bottom"]) / 2.0
-
     def _get_bottom_line_width(self):
         """Returns width of bottom line"""
 
         return float(self.cell_attributes[self.key]["borderwidth_bottom"]) / 2.
 
-    def _get_left_line_width(self):
-        """Returns width of left line"""
-
-        row, col, tab = self.key
-        key = row, col - 1, tab
-        return float(self.cell_attributes[key]["borderwidth_right"]) / 2.0
-
     def _get_right_line_width(self):
         """Returns width of right line"""
 
         return float(self.cell_attributes[self.key]["borderwidth_right"]) / 2.0
 
-    def _get_topleft_right_line_width(self):
-        """Returns width of right line of top left cell"""
+    def get_b(self):
+        """Returns the bottom border of the cell"""
 
-        row, col, tab = self.key
-        key = row - 1, col - 1, tab
-        return float(self.cell_attributes[key]["borderwidth_right"]) / 2.0
+        start_point, end_point = self._get_bottom_line_coordinates()
+        width = self._get_bottom_line_width()
+        color = self._get_bottom_line_color()
 
-    def _get_topleft_bottom_line_width(self):
-        """Returns width of bottom line of top left cell"""
+        return CellBorder(start_point, end_point, width, color)
 
-        row, col, tab = self.key
-        key = row - 1, col - 1, tab
-        return float(self.cell_attributes[key]["borderwidth_bottom"]) / 2.0
+    def get_r(self):
+        """Returns the right border of the cell"""
 
-    # Drawing
-    # -------
+        start_point, end_point = self._get_right_line_coordinates()
+        width = self._get_right_line_width()
+        color = self._get_right_line_color()
 
-    def _draw_line(self, start_point, end_point):
-        """Draws a line without setting color or width
+        return CellBorder(start_point, end_point, width, color)
 
-        Parameters
-        ----------
-        start_point: 2 tuple of Integer
-        \tStart point of line
-        end_point
-        \tEnd point of line
+    def get_t(self):
+        """Returns the top border of the cell"""
 
-        """
+        cell_above = CellBorders(self.cell_attributes,
+                                 *self.cell.get_above_key_rect())
+        return cell_above.get_b()
 
-        self.context.move_to(*start_point)
-        self.context.line_to(*end_point)
-        self.context.stroke()
+    def get_l(self):
+        """Returns the left border of the cell"""
 
-    def _draw_left_line(self):
-        """Draws left line setting color and width"""
+        cell_left = CellBorders(self.cell_attributes,
+                                *self.cell.get_left_key_rect())
+        return cell_left.get_r()
 
-        self.context.set_source_rgb(*self._get_left_line_color())
-        self.context.set_line_width(self._get_left_line_width())
-        self._draw_line(*self._get_left_line_coordinates())
+    def get_tl(self):
+        """Returns the top left border of the cell"""
 
-    def _draw_right_line(self):
-        """Draws right line setting color and width"""
+        cell_above_left = CellBorders(self.cell_attributes,
+                                      *self.cell.get_above_left_key_rect())
+        return cell_above_left.get_r()
 
-        self.context.set_source_rgb(*self._get_right_line_color())
-        self.context.set_line_width(self._get_right_line_width())
-        self._draw_line(*self._get_right_line_coordinates())
+    def get_tr(self):
+        """Returns the top right border of the cell"""
 
-    def _draw_top_line(self):
-        """Draws top line setting color and width"""
+        cell_above = CellBorders(self.cell_attributes,
+                                 *self.cell.get_above_key_rect())
+        return cell_above.get_r()
 
-        self.context.set_source_rgb(*self._get_top_line_color())
-        self.context.set_line_width(self._get_top_line_width())
-        self._draw_line(*self._get_top_line_coordinates())
+    def get_rt(self):
+        """Returns the right top border of the cell"""
 
-    def _draw_bottom_line(self):
-        """Draws bottom line setting color and width"""
+        cell_above_right = CellBorders(self.cell_attributes,
+                                       *self.cell.get_above_right_key_rect())
+        return cell_above_right.get_b()
 
-        self.context.set_source_rgb(*self._get_bottom_line_color())
-        self.context.set_line_width(self._get_bottom_line_width())
-        self._draw_line(*self._get_bottom_line_coordinates())
+    def get_rb(self):
+        """Returns the right bottom border of the cell"""
 
-    def _draw_tlr_line(self):
-        """Draws bottom line setting color and width"""
+        cell_right = CellBorders(self.cell_attributes,
+                                 *self.cell.get_right_key_rect())
+        return cell_right.get_b()
 
-        self.context.set_source_rgb(*self._get_topleft_right_line_color())
-        self.context.set_line_width(self._get_topleft_right_line_width())
-        self._draw_line(*self._get_topleft_right_line_coordinates())
+    def get_br(self):
+        """Returns the bottom right border of the cell"""
 
-    def _draw_tlb_line(self):
-        """Draws bottom line setting color and width"""
+        cell_below = CellBorders(self.cell_attributes,
+                                 *self.cell.get_below_key_rect())
+        return cell_below.get_r()
 
-        self.context.set_source_rgb(*self._get_topleft_bottom_line_color())
-        self.context.set_line_width(self._get_topleft_bottom_line_width())
-        self._draw_line(*self._get_topleft_bottom_line_coordinates())
+    def get_bl(self):
+        """Returns the bottom left border of the cell"""
+
+        cell_below_left = CellBorders(self.cell_attributes,
+                                      *self.cell.get_below_left_key_rect())
+        return cell_below_left.get_r()
+
+    def get_lb(self):
+        """Returns the left bottom border of the cell"""
+
+        cell_left = CellBorders(self.cell_attributes,
+                                *self.cell.get_left_key_rect())
+        return cell_left.get_b()
+
+    def get_lt(self):
+        """Returns the left top border of the cell"""
+
+        cell_above_left = CellBorders(self.cell_attributes,
+                                      *self.cell.get_above_left_key_rect())
+        return cell_above_left.get_b()
+
+    def gen_all(self):
+        """Generator of all borders"""
+
+        borderfuncs = [
+            self.get_b, self.get_r, self.get_t, self.get_l,
+            self.get_tl, self.get_tr, self.get_rt, self.get_rb,
+            self.get_br, self.get_bl, self.get_lb, self.get_lt,
+        ]
+
+        for borderfunc in borderfuncs:
+            yield borderfunc()
+
+
+class GridCellBorderCairoRenderer(object):
+    """Renders cell border to Cairo context
+
+    Parameters
+    ----------
+
+    * context: pycairo.context
+    \tThe Cairo context to be drawn to
+    * code_array: model.code_array
+    \tGrid data structure that yields rendering information
+    * key: 3-tuple of Integer
+    \tKey of cell to be rendered
+
+    """
+
+    def __init__(self, context, code_array, key, rect):
+        self.context = context
+        self.code_array = code_array
+        self.cell_attributes = code_array.cell_attributes
+        self.key = key
+        self.rect = rect
 
     def draw(self):
         """Draws cell border to context"""
@@ -964,23 +1228,14 @@ class GridCellBorderCairoRenderer(object):
         # Lines should have a square cap to avoid ugly edges
         self.context.set_line_cap(cairo.LINE_CAP_SQUARE)
 
-        # Sort lines from thinnest to thickest
-        width_func = [
-            (self._get_left_line_width(),
-             -sum(self._get_left_line_color()), self._draw_left_line),
-            (self._get_right_line_width(),
-             -sum(self._get_right_line_color()), self._draw_right_line),
-            (self._get_top_line_width(),
-             -sum(self._get_top_line_color()), self._draw_top_line),
-            (self._get_bottom_line_width(),
-             -sum(self._get_bottom_line_color()), self._draw_bottom_line),
-            (self._get_topleft_right_line_width(),
-             -sum(self._get_topleft_right_line_color()), self._draw_tlr_line),
-            (self._get_topleft_bottom_line_width(),
-             -sum(self._get_topleft_bottom_line_color()), self._draw_tlb_line),
-        ]
+        self.context.save()
+        self.context.rectangle(*self.rect)
+        self.context.clip()
 
-        width_func.sort()
+        cell_borders = CellBorders(self.cell_attributes, self.key, self.rect)
+        borders = list(cell_borders.gen_all())
+        borders.sort(key=attrgetter('width', 'color'))
+        for border in borders:
+            border.draw(self.context)
 
-        for _, _, draw_func in width_func:
-            draw_func()
+        self.context.restore()
